<?

  /***************************************************************************\
  *     All rights reversed - Copyleft 2005 - Originally by Markku Uttula     *
  *                                                                           *
  *                   http://www.disconova.com/open_source/                   *
  *---------------------------------------------------------------------------*
  *  PLEASE READ THIS BEFORE ATTEMPTING TO USE THESE CLASSES, THANK YOU :)    *
  *---------------------------------------------------------------------------*
  *  The classes in this file, pop3session and pop3message rely on the mail   *
  *  server complying to POP3-protocol as specified in RFC-1939. To be more   *
  *  specific, it needs to support the "Minimal POP3 Commands" (well - they   *
  *  all should:) as well as the "Optional POP3 Commands" UIDL and TOP.       *
  *                                                                           *
  *  Not all servers (but most of them) support UIDL- and/or TOP-commands.    *
  *  If this is the case with your server, I'm only too sorry, but these      *
  *  classes *will not work*. They really do rely on those commands.          *
  *                                                                           *
  *  Thanks for listening, blue skies...        Markku Uttula, Jan 27th 2005  *
  \***************************************************************************/

  /***************************************************************************\
  * pop3session                                                               *
  * ===========                                                               *
  *                                                                           *
  *   This is the object used to handle a single POP3 server/client-          *
  *   session. All functions (except the constructor) return boolean          *
  *   FALSE if they fail.                                                     *
  *                                                                           *
  * - pop3session ( $server[:port], $username, $userpass );                   *
  *                                                                           *
  *   Object constructor. Variable $server is the host (with the port         *
  *   as optional addition) to connect to, $username and $userpass            *
  *   are the credentials used when opening the session.                      *
  *                                                                           *
  * - object->open ( [$timeout] );                                            *
  *                                                                           *
  *   Opens a session with a POP3-server. If provided with an optional        *
  *   variable $timeout, will only try for given amount of seconds for        *
  *   the connection to be established.                                       *
  *                                                                           *
  * - object->close ();                                                       *
  *                                                                           *
  *   Closes a session. Messages marked for deletion will actually be         *
  *   removed from the server after this.                                     *
  *                                                                           *
  * - object->reset ();                                                       *
  *                                                                           *
  *   Resets the mailbox. Messages that were marked for deletion but          *
  *   not actually removed will be available after this (and during           *
  *   following sessions until they're marked for deletion and actually       *
  *   removed from the server).                                               *
  *                                                                           *
  * - object->error ();                                                       *
  *                                                                           *
  *   Returns a textual explanation of the last error that occurred.          *
  *                                                                           *
  * - object->messages                                                        *
  *                                                                           *
  *   This is an array of pop3message-objects. Array is empty unless          *
  *   session is open. You can use count($object->messages) to find           *
  *   out how many messages there are in the mailbox. You can also            *
  *   loop through the pop3message-objects using something like the           *
  *   following:                                                              *
  *                                                                           *
  *   <?                                                                      *
  *       foreach (array_keys($object->messages) as $msg_id) {                *
  *         $message = &$object->messages[$msg_id];                           *
  *       }                                                                   *
  *   ?>                                                                      *
  *                                                                           *
  \***************************************************************************/

  /***************************************************************************\
  * pop3message                                                               *
  * ===========                                                               *
  *                                                                           *
  *   This is the object used to handle a single message within a             *
  *   mailbox opened using pop3session-object. This object should             *
  *   never be created from outside pop3session... It has a specific          *
  *   purpose, and is of no help outside the context of pop3session.          *
  *   All functions return a boolean FALSE if they fail (if this              *
  *   happens, you can use pop3session-object's error()-function for          *
  *   a textual explanation of the error that occurred.                       *
  *                                                                           *
  * - object->messages[n]->uid();                                             *
  *                                                                           *
  *   Returns the UID (Universal IDentifier) for this message. As per         *
  *   RFC-1939, UIDs *should* uniquely identify a message within a            *
  *   maildrop and persist across sessions. If this isn't the case with       *
  *   your mailserver, blame the developers of the server software for        *
  *   not doing their job properly. Also, see note on the beginning of        *
  *   this file.                                                              *
  *                                                                           *
  * - object->messages[n]->delete ();                                         *
  *                                                                           *
  *   Marks a message for deletion. After message has marked, you             *
  *   can't retrieve header or source of that message. However, you           *
  *   can use the pop3session-object's reset()-function to remove             *
  *   marking from *all* messages. Sorry for this - I wasn't the one          *
  *   who designed POP3-protocol. Blame them :)                               *
  *                                                                           *
  * - object->messages[n]->header ();                                         *
  *                                                                           *
  *   Returns the main headers of the message. Please note that the           *
  *   headers aren't retrieved from the server before asking for them.        *
  *                                                                           *
  * - object->messages[n]->source ();                                         *
  *                                                                           *
  *   Returns the full source of the message. Please note that the            *
  *   source isn't retrieved from the server before asking for them.          *
  *                                                                           *
  \***************************************************************************/

  class pop3session {

    var $_server_socket;
    var $_server_host;
    var $_server_port;
    var $_server_user;
    var $_server_pass;
    var $_last_error;
    var $messages;

    function pop3session ($server, $name, $pass) {
      $this->messages = array();
      $this->_server_user = $name;
      $this->_server_pass = $pass;
      if (strpos($server, ':') === FALSE) {
        $this->_server_port = 110; // Default port for POP3
        $this->_server_host = $server;
      } else {
        // You can set the port to use by assigning $server-variable into
        // something like "www.mailserver.net:1234" where 1234 is the port
        $this->_server_port = substr($server, strpos($server, ':') + 1, strlen($server));
        $this->_server_host = substr($server, 0, strpos($server, ':'));
      }
    }

    function _rx($multiline = FALSE) {
      if ($multiline) {
        $etb = "\r\n".'.'."\r\n";
      } else {
        $etb = "\r\n";
      }
      $rx = '';
      $in = TRUE;
      while ((substr($rx, -(strlen($etb))) !== $etb) && ($in)) {
        $in = @fgets($this->_server_socket, 4096);
        $rx .= $in;
      }
      if (!$in) {
        $this->_last_error = 'Socket error: Unable to read from socket.';
        $rx = FALSE;
      } elseif (substr($rx, 0, 3) !== '+OK') {
        $this->_last_error = 'Protocol error: Server responded "'.$rx.'".';
        $rx = FALSE;
      }
      return $rx;
    }

    function _tx($data) {
      if (fwrite($this->_server_socket, $data."\r\n") !== FALSE) {
        $result = TRUE;
      } else {
        $result = FALSE;
      }
      return $result;
    }

    function error() {
      if ($this->_last_error !== '') {
        return $this->_last_error;
      } else {
        return 'No error.';
      }
    }

    function reset() {
      $this->_tx('RSET');
      if (!$this->_rx()) {
        $result = FALSE;
      } else {
        foreach (array_keys($this->messages) as $msg_id) {
          $this->messages[$msg_id]->_deleted = FALSE;
        }
        $result = TRUE;
      }
    }

    function open($timeout = 60) {
      $this->_last_error = '';
      $error_string = '';
      $error_number = -1;
      $this->_server_socket = @fsockopen($this->_server_host, $this->_server_port, $error_number, $error_string, $timeout);
      if (!$this->_server_socket) {
        $this->_last_error = 'Socket error #'.$error_number.': '.$error_string;
        $result = FALSE;
      } else {
        $result = $this->_rx();
        if ($result && !$this->_tx('USER '.$this->_server_user)) {
          $result = FALSE;
        } else {
          $result = $this->_rx();
        }
        if ($result && !$this->_tx('PASS '.$this->_server_pass)) {
          $result = FALSE;
        } else {
          $result = $this->_rx();
        }
        if ($result && !$this->_tx('UIDL')) {
          $result = FALSE;
        } else {
          $result = $this->_rx(TRUE);
          if ($result) {
            $uidl = explode("\r\n", $result);
            for ($i = 1; $i < count($uidl) - 2; $i++) {
              $this->messages[$i-1] = new pop3message($i, substr($uidl[$i], strpos($uidl[$i], ' ') + 1), $this);
            }
          }
        }
      }
      return $result;
    }

    function close() {
      $this->_tx('QUIT');
      if (!$this->_rx()) {
        fclose($this->_server_socket);
        $this->messages = array();
        $result = FALSE;
      } else {
        $result = TRUE;
      }
      return $result;
    }

  }

  
  class pop3message {
 
    var $_session;
    var $_deleted;
    var $_id;
    var $_header;
    var $_source;
    var $_uid;

    function pop3message($id, $uid, &$session) {
      $this->_session = $session;
      $this->_deleted = FALSE;
      $this->_id = $id;
      $this->_uid = $uid;
      $this->_header = '';
      $this->_source = '';
    }

    function uid() {
      return $this->_uid;
    }
    
    function delete() {
      if (!$this->_deleted) {
        $this->_session->_tx('DELE '.$this->_id);
        if (!$this->_session->_rx()) {
          $result = FALSE;
        } else {
          $this->_deleted = TRUE;
          $result = TRUE;
        }
      } else {
        $result = TRUE;
      }
      return $result;
    }
    
    function header() {
      if ($this->_deleted) {
        // $this->_last_error = 'Object error: This message is marked for deletion and can not be retrieved.';
        $result = FALSE;
      } else {
        if ($this->_header === '') {
          $this->_session->_tx('TOP '.$this->_id.' 0');
          $this->_header = $this->_session->_rx(TRUE);
          if (!$this->_header) {
            $this->_header = '';
            $result = FALSE;
          } else {
            $this->_header = explode("\r\n", $this->_header);
            array_shift($this->_header);
            array_pop($this->_header);
            array_pop($this->_header);
            $this->_header = implode("\r\n", $this->_header);
            $result = $this->_header;
          }
        } else {
          $result = $this->_header;
        }
      }
      return $result;
    }

    function source() {
      if ($this->_deleted) {
        // $this->_last_error = 'Object error: This message is marked for deletion and can not be retrieved.';
        $result = FALSE;
      } else {
        if ($this->_source === '') {
          $this->_session->_tx('RETR '.$this->_id);
          $this->_source = $this->_session->_rx(TRUE);
          if (!$this->_source) {
            $this->_source = '';
            $result = FALSE;
          } else {
            $this->_source = explode("\r\n", $this->_source);
            array_shift($this->_source);
            array_pop($this->_source);
            array_pop($this->_source);
            $this->_source = implode("\r\n", $this->_source);
            $result = $this->_source;
          }
        } else {
          $result = $this->_source;
        }
      }
      return $result;
    }
    
  }
?>